"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.converge = exports.truncate = exports.syncSchema = exports.dropSchema = void 0;
const debug_1 = __importDefault(require("debug"));
const migrate_1 = require("./migrate");
const debug = (0, debug_1.default)('hypermodel:converge');
const dropSchema = ({ logger, connection, }) => __awaiter(void 0, void 0, void 0, function* () {
    const { yellow, bold } = logger.colors;
    logger.info(`${bold(yellow('models:'))} dropping schema`);
    yield connection.query(`DROP TABLE IF EXISTS "SequelizeMeta";`);
    yield connection.drop();
});
exports.dropSchema = dropSchema;
const syncSchema = ({ logger, connection, }) => __awaiter(void 0, void 0, void 0, function* () {
    const { yellow, bold } = logger.colors;
    logger.info(`${bold(yellow('models:'))} syncing schema`);
    yield connection.sync({ force: false });
});
exports.syncSchema = syncSchema;
const truncate = ({ logger, connection, models, }) => __awaiter(void 0, void 0, void 0, function* () {
    const { yellow, bold } = logger.colors;
    logger.info(`${bold(yellow('models:'))} truncating tables`);
    for (const model of Object.values(models)) {
        const m = model;
        yield m.truncate({ restartIdentity: true, cascade: true, force: true });
        // https://github.com/sequelize/sequelize/issues/11152
        if (connection.getDialect() === 'sqlite') {
            yield connection.query('PRAGMA foreign_keys = OFF;');
            yield connection.query(`DELETE FROM ${m.tableName};`);
            yield connection.query(`DELETE FROM sqlite_sequence;`);
        }
    }
});
exports.truncate = truncate;
const converge = ({ migrationsRoot, models, config, connection, logger, }) => __awaiter(void 0, void 0, void 0, function* () {
    debug('configuration %o', config);
    if (config.dropSchema) {
        yield (0, exports.dropSchema)({ logger, connection });
    }
    if (config.synchronize) {
        yield (0, exports.syncSchema)({ logger, connection });
    }
    if (config.migrate) {
        const mig = (0, migrate_1.migrate)('auto');
        yield mig({ migrationsRoot, connection, logger });
    }
    if (config.truncate) {
        yield (0, exports.truncate)({ logger, connection, models });
    }
});
exports.converge = converge;
