"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tableBuilder = exports.migrate = void 0;
/* eslint-disable @typescript-eslint/no-var-requires */
const sequelize_1 = require("sequelize");
const lodash_1 = __importDefault(require("lodash"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('hypermodel:migrate');
const tableBuilder = {
    build: (...args) => lodash_1.default.reduce(args, lodash_1.default.merge, {}),
    uniqueConstraint: (name, fields, opts) => (Object.assign({ fields, type: 'unique', name }, opts)),
    intFK: (name, toTable, toKey, opts) => ({
        [name]: Object.assign({ type: sequelize_1.INTEGER, allowNull: false, references: {
                model: toTable,
                key: toKey,
            } }, opts),
    }),
    intPK: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.INTEGER, allowNull: false, autoIncrement: true, primaryKey: true }, opts),
    }),
    json: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.JSON }, opts),
    }),
    text: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.TEXT }, opts),
    }),
    string: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.STRING }, opts),
    }),
    nonNullString: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.STRING, allowNull: false }, opts),
    }),
    uniqueString: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.STRING, allowNull: false, unique: true }, opts),
    }),
    date: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.DATE }, opts),
    }),
    nonNullDate: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.DATE, allowNull: false }, opts),
    }),
    bool: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.BOOLEAN }, opts),
    }),
    int: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.INTEGER }, opts),
    }),
    nonNullInt: (name, opts) => ({
        [name]: Object.assign({ type: sequelize_1.INTEGER, allowNull: false }, opts),
    }),
    timestamps: (opts) => ({
        createdAt: Object.assign({ type: sequelize_1.DATE, allowNull: false }, opts),
        updatedAt: Object.assign({ type: sequelize_1.DATE, allowNull: false }, opts),
    }),
};
exports.tableBuilder = tableBuilder;
// NOTE: 'auto' is the only operational value, the rest are there for future implementation
// commands which are not 'auto' are assumed to be run from CLI, 'auto' is part of
// auto migration, and is always in process when the app is starting
// the opposite of auto is 'pending'. might want to change it to 'invoke' or something
const logTransform = ({ event, name, durationSeconds, }) => {
    if (event === 'migrating') {
        return `===>    up   | ${name}`;
    }
    else if (event === 'migrated') {
        return `             | ${name} (in ${durationSeconds}s)`;
    }
    else if (event === 'reverting') {
        return `===>    down | ${name}`;
    }
    else if (event === 'reverted') {
        return `             | ${name} (in ${durationSeconds}s)`;
    }
    else {
        return `${event}: ${name} (tbd)`;
    }
};
const migrate = (cmd, _id) => ({ migrationsRoot, logger, connection, }) => __awaiter(void 0, void 0, void 0, function* () {
    const { Umzug, SequelizeStorage } = require('umzug');
    const migrationsPath = migrationsRoot;
    debug('migration root is:', migrationsPath);
    const storage = new SequelizeStorage({
        sequelize: connection,
    });
    const umzug = new Umzug({
        storage,
        logger: Object.assign(Object.assign({}, logger), { info: (msg) => logger.info(logTransform(msg)) }),
        migrations: {
            glob: migrationsPath,
            resolve: ({ name, path, context }) => {
                // Adjust the migration from the new signature to the v2 signature, making easier to upgrade to v3
                const migration = require(path);
                const newContext = Object.assign(Object.assign({}, context), { query: connection.getQueryInterface(), t: tableBuilder });
                return {
                    name,
                    up: () => __awaiter(void 0, void 0, void 0, function* () { return yield migration.up(newContext); }),
                    down: () => __awaiter(void 0, void 0, void 0, function* () { return yield migration.down(newContext); }),
                };
            },
        },
    });
    const { yellow, bold } = logger.colors;
    const pref = bold(yellow('models:'));
    // Checks migrations and run them if they are not already applied. To keep
    // track of the executed migrations, a table (and sequelize model) called SequelizeMeta
    // will be automatically created (if it doesn't exist already) and parsed.
    logger.info(`${pref} === migrations: start ===`);
    yield umzug.up();
    logger.info(`${pref} === migrations: done  ===`);
    if (cmd !== 'auto') {
        // we are running this from CLI, other
        // wise keep the connection because it runs inprocess as part
        // of auto migration
        yield connection.close();
    }
});
exports.migrate = migrate;
