"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildModels = void 0;
const glob_1 = require("glob");
const sequelize_typescript_1 = require("sequelize-typescript");
const lodash_1 = __importDefault(require("lodash"));
const load = (patt) => (0, glob_1.globSync)(patt).map((f) => {
    let mod = null;
    try {
        mod = require(f);
    }
    catch (e) {
        console.log(`error dynamically loading '${f}'\n${e.toString()}`); // eslint-disable-line no-console
        throw e;
    }
    return mod.default || mod;
});
const isTrue = (val) => val === true || val === 'true';
const POOL_MAX_DEFAULT = 5;
const POOL_MIN_DEFAULT = 1;
const POOL_ACQUIRE_DEFAULT = 30000;
const POOL_IDLE_DEFAULT = 10000;
const createSequelizeConfig = ({ logger, config, modelsRoot, }) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d;
    const models = lodash_1.default.reduce(load(modelsRoot), lodash_1.default.merge);
    const max_pool_connection = ((_a = config.pool) === null || _a === void 0 ? void 0 : _a.max) || POOL_MAX_DEFAULT;
    let min_pool_connection = ((_b = config.pool) === null || _b === void 0 ? void 0 : _b.min) || POOL_MIN_DEFAULT;
    if (min_pool_connection > max_pool_connection) {
        logger.error(`max_pool_connection: ${max_pool_connection} is smaller than min_pool_connection: ${min_pool_connection}. setting min=max`);
        min_pool_connection = max_pool_connection;
    }
    let modelList = [];
    // bun.js
    modelList = Object.values(Object.getPrototypeOf(models));
    if (modelList.length === 0) {
        // node.js
        modelList = Object.values(models);
    }
    const uri = config.uri;
    return {
        uri,
        models,
        sequelizeConfig: {
            logging: config.logging,
            benchmark: isTrue(config.logging),
            native: config.native,
            ssl: config.ssl,
            models: modelList,
            pool: {
                max: max_pool_connection,
                min: min_pool_connection,
                acquire: ((_c = config.pool) === null || _c === void 0 ? void 0 : _c.acquire) || POOL_ACQUIRE_DEFAULT,
                idle: ((_d = config.pool) === null || _d === void 0 ? void 0 : _d.idle) || POOL_IDLE_DEFAULT,
            },
        },
    };
});
const buildModels = ({ logger, config, modelsRoot, }) => __awaiter(void 0, void 0, void 0, function* () {
    const { uri, sequelizeConfig, models } = yield createSequelizeConfig({
        logger,
        config,
        modelsRoot,
    });
    const connection = new sequelize_typescript_1.Sequelize(uri, sequelizeConfig);
    return { models, connection };
});
exports.buildModels = buildModels;
